@@ -4,20 +4,18 @@ import functools
 import threading
 import time
 import uuid
-from collections import defaultdict
 from collections.abc import Callable, Sequence
 from contextlib import contextmanager
-from typing import Any, Protocol, cast, runtime_checkable
+from typing import TYPE_CHECKING, Any, Protocol, cast, runtime_checkable
 
 import pyarrow as pa
 import sympy as sp
 
 from dqx import functions, get_logger
-from dqx.analyzer import AnalysisReport, Analyzer, MetricKey
+from dqx.analyzer import AnalysisReport, Analyzer
 from dqx.common import (
     AssertionResult,
     DQXError,
-    Metadata,
     ResultKey,
     SeverityLevel,
     SqlDataSource,
@@ -26,13 +24,16 @@ from dqx.common import (
 from dqx.evaluator import Evaluator
 from dqx.graph.nodes import CheckNode, RootNode
 from dqx.graph.traversal import Graph
-from dqx.orm.repositories import MetricDB
+
+# import moved to local scope(s) to avoid cyclic dependency
 from dqx.plugins import PluginExecutionContext, PluginManager
 from dqx.provider import MetricProvider, SymbolicMetric
-from dqx.specs import MetricSpec
 from dqx.timer import Registry
 from dqx.validator import SuiteValidator
 
+if TYPE_CHECKING:
+    from dqx.orm.repositories import MetricDB, MetricStats
+
 CheckProducer = Callable[[MetricProvider, "Context"], None]
 CheckCreator = Callable[[CheckProducer], CheckProducer]
 
@@ -202,7 +203,7 @@ class Context:
     graph nodes that need access to the symbol table.
     """
 
-    def __init__(self, suite: str, db: MetricDB, execution_id: str) -> None:
+    def __init__(self, suite: str, db: "MetricDB", execution_id: str) -> None:
         """
         Initialize the context with a root graph node.
 
@@ -212,6 +213,7 @@ class Context:
             execution_id: Unique identifier for this execution
         """
         self._graph = Graph(RootNode(name=suite))
+        # MetricProvider now creates its own cache internally
         self._provider = MetricProvider(db, execution_id=execution_id)
         self._local = threading.local()
 
@@ -319,7 +321,7 @@ class VerificationSuite:
     def __init__(
         self,
         checks: Sequence[CheckProducer | DecoratedCheck],
-        db: MetricDB,
+        db: "MetricDB",
         name: str,
     ) -> None:
         """
@@ -361,7 +363,10 @@ class VerificationSuite:
         self._analyze_ms = timer_registry.timer("analyzing.time_ms")
 
         # Store analysis reports by datasource name
-        self._analysis_reports: dict[str, AnalysisReport] = {}
+        self._analysis_reports: AnalysisReport
+
+        # Cache for metrics stats
+        self._metrics_stats: "MetricStats | None" = None
 
     @property
     def execution_id(self) -> str:
@@ -403,7 +408,7 @@ class VerificationSuite:
         return self._context._graph
 
     @property
-    def analysis_reports(self) -> dict[str, AnalysisReport]:
+    def analysis_reports(self) -> AnalysisReport:
         """
         Access the analysis reports generated by the suite.
 
@@ -431,6 +436,22 @@ class VerificationSuite:
         """
         return self._context.provider
 
+    @property
+    def metrics_stats(self) -> "MetricStats":
+        """
+        Get the cached metrics statistics.
+
+        Returns:
+            MetricStats instance containing total and expired metric counts
+
+        Raises:
+            DQXError: If accessed before the suite has been run
+        """
+        self.assert_is_evaluated()
+        if self._metrics_stats is None:
+            raise DQXError("Metrics stats not available. This should not happen after successful run().")
+        return self._metrics_stats
+
     @property
     def plugin_manager(self) -> PluginManager:
         """
@@ -517,44 +538,11 @@ class VerificationSuite:
             logger.debug(f"Suite validation warnings:\n{report}")
 
     def _analyze(self, datasources: list[SqlDataSource], key: ResultKey) -> None:
-        # Analyze ALL symbolic metrics, not just those with matching dataset
-        all_symbolic_metrics = self._context.provider.metrics
+        """Analyze datasources using the new analyze_all function."""
 
-        # Group metrics by dataset (including None for unassigned)
-        metrics_by_dataset: dict[str | None, list[SymbolicMetric]] = defaultdict(list)
-        for sym_metric in all_symbolic_metrics:
-            metrics_by_dataset[sym_metric.dataset].append(sym_metric)
-
-        for ds in datasources:
-            # Get metrics that either match this dataset or have no dataset assigned
-            relevant_metrics = metrics_by_dataset.get(ds.name, []) + metrics_by_dataset.get(None, [])
-
-            # Skip if no metrics for this dataset
-            if not relevant_metrics:
-                continue
-
-            # Create symbol lookup dictionary using MetricKey
-            symbol_lookup: dict[MetricKey, str] = {}
-
-            # Group metrics by their effective date
-            metrics_by_date: dict[ResultKey, list[MetricSpec]] = defaultdict(list)
-            for sym_metric in relevant_metrics:
-                # Use lag directly instead of key_provider
-                effective_key = key.lag(sym_metric.lag)
-                metrics_by_date[effective_key].append(sym_metric.metric_spec)
-                # Add to symbol lookup with MetricKey including dataset
-                symbol_lookup[(sym_metric.metric_spec, effective_key, ds.name)] = str(sym_metric.symbol)
-
-            # Analyze each date group separately
-            metadata = Metadata(execution_id=self._execution_id)
-            analyzer = Analyzer(metadata=metadata, symbol_lookup=symbol_lookup)
-            analyzer.analyze(ds, metrics_by_date)
-
-            # Persist the combined report
-            analyzer.report.persist(self.provider._db)
-
-            # Store the report for later access
-            self._analysis_reports[ds.name] = analyzer.report
+        # Call analyze_all and store the results
+        analyzer = Analyzer(datasources, self.provider, key, execution_id=self._execution_id)
+        self._analysis_reports = analyzer.analyze()
 
     def run(self, datasources: list[SqlDataSource], key: ResultKey, *, enable_plugins: bool = True) -> None:
         """
@@ -600,6 +588,18 @@ class VerificationSuite:
         # Apply symbol deduplication BEFORE analysis
         self._context.provider.symbol_deduplication(self._context._graph, key)
 
+        # Collect metrics stats and cleanup expired metrics BEFORE analysis
+        self._metrics_stats = self.provider._db.get_metrics_stats()
+        logger.info(
+            f"Metrics stats: {self._metrics_stats.expired_metrics} expired "
+            f"out of {self._metrics_stats.total_metrics} total"
+        )
+
+        # Cleanup expired metrics before analysis
+        if self._metrics_stats.expired_metrics > 0:
+            logger.info("Cleaning up expired metrics...")
+            self.cleanup_expired_metrics()
+
         # 2. Analyze by datasources
         with self._analyze_ms:
             self._analyze(datasources, key)
@@ -716,7 +716,7 @@ class VerificationSuite:
 
         return False
 
-    def metric_trace(self, db: MetricDB) -> pa.Table:
+    def metric_trace(self, db: "MetricDB") -> pa.Table:
         """
         Generate a metric trace table showing how metrics flow through the system.
 
@@ -761,6 +761,7 @@ class VerificationSuite:
             execution_id=self.execution_id,
             reports=self._analysis_reports,
             symbols=symbols,
+            symbol_lookup=self.provider.registry.symbol_lookup_table(self.key),
         )
 
     def _process_plugins(self, datasources: list[SqlDataSource]) -> None:
@@ -785,11 +786,29 @@ class VerificationSuite:
             results=self.collect_results(),
             symbols=self.provider.collect_symbols(self.key),
             trace=self.metric_trace(self.provider._db),
+            metrics_stats=self.metrics_stats,
+            cache_stats=self.provider.cache.get_stats(),
         )
 
         # Process through all plugins
         self.plugin_manager.process_all(context)
 
+    def cleanup_expired_metrics(self) -> None:
+        """
+        Delete expired metrics from the database.
+
+        This method should be called periodically to remove metrics that have
+        exceeded their TTL (time-to-live). It uses UTC time for consistency
+        across different timezones.
+
+        Example:
+            >>> suite = VerificationSuite(checks, db, "My Suite")
+            >>> # After running the suite...
+            >>> suite.cleanup_expired_metrics()
+            >>> print("Expired metrics deleted")
+        """
+        self.provider._db.delete_expired_metrics()
+
 
 def _create_check(
     provider: MetricProvider,
