@@ -10,14 +10,18 @@ from threading import Lock
 from typing import TYPE_CHECKING, Callable, overload
 
 import sympy as sp
+from returns.maybe import Some
 from returns.result import Failure, Result
 
 from dqx import compute, specs
-from dqx.common import DQXError, ExecutionId, ResultKey, RetrievalFn, Tags
+from dqx.cache import MetricCache
+from dqx.common import DQXError, ExecutionId, MetricKey, ResultKey, RetrievalFn, Tags
+from dqx.models import Metric
 from dqx.orm.repositories import MetricDB
 from dqx.specs import MetricSpec
 
 if TYPE_CHECKING:
+    from dqx.cache import MetricCache
     from dqx.graph.traversal import Graph
 
 logger = logging.getLogger(__name__)
@@ -61,22 +65,7 @@ class SymbolicMetric:
 
 
 def _create_lazy_retrieval_fn(provider: "MetricProvider", metric_spec: MetricSpec, symbol: sp.Symbol) -> RetrievalFn:
-    """Create a lazy retrieval function that resolves dataset at evaluation time.
-
-    This factory creates a retrieval function that defers dataset resolution
-    until the metric is actually evaluated. This allows metrics to be created
-    before their dataset is known (during imputation), while ensuring the
-    correct dataset is used during evaluation.
-
-    Args:
-        provider: The MetricProvider instance containing the symbol registry.
-        metric_spec: The metric specification to evaluate.
-        symbol: The symbol representing this metric.
-
-    Returns:
-        A retrieval function that looks up the dataset from the SymbolicMetric
-        at evaluation time and uses it to fetch the correct metric value.
-    """
+    """Create retrieval function with deferred dataset resolution."""
 
     def lazy_retrieval_fn(key: ResultKey) -> Result[float, str]:
         # Look up the current dataset from the SymbolicMetric
@@ -89,31 +78,18 @@ def _create_lazy_retrieval_fn(provider: "MetricProvider", metric_spec: MetricSpe
             return Failure(f"Dataset not imputed for metric {symbolic_metric.name}")
 
         # Call the compute function with the resolved dataset and execution_id
-        return compute.simple_metric(provider._db, metric_spec, symbolic_metric.dataset, key, provider.execution_id)
+        return compute.simple_metric(metric_spec, symbolic_metric.dataset, key, provider.execution_id, provider._cache)
 
     return lazy_retrieval_fn
 
 
 def _create_lazy_extended_fn(
     provider: "MetricProvider",
-    compute_fn: Callable[[MetricDB, MetricSpec, str, ResultKey, ExecutionId], Result[float, str]],
+    compute_fn: Callable[..., Result[float, str]],
     metric_spec: MetricSpec,
     symbol: sp.Symbol,
 ) -> RetrievalFn:
-    """Create a lazy retrieval function for extended metrics (DoD, WoW, etc).
-
-    Similar to _create_lazy_retrieval_fn but for extended metrics that
-    need to call specialized compute functions.
-
-    Args:
-        provider: The MetricProvider instance.
-        compute_fn: The compute function to use (e.g., compute.day_over_day).
-        metric_spec: The base metric specification.
-        symbol: The symbol representing this metric.
-
-    Returns:
-        A lazy retrieval function for the extended metric.
-    """
+    """Create retrieval function for extended metrics with deferred dataset resolution."""
 
     def lazy_extended_fn(key: ResultKey) -> Result[float, str]:
         # Look up the current dataset
@@ -126,7 +102,8 @@ def _create_lazy_extended_fn(
             return Failure(f"Dataset not imputed for metric {symbolic_metric.name}")
 
         # Call the compute function with the resolved dataset and execution_id
-        return compute_fn(provider._db, metric_spec, symbolic_metric.dataset, key, provider.execution_id)
+        # Note: compute_fn may be a lambda that already includes additional parameters
+        return compute_fn(metric_spec, symbolic_metric.dataset, key, provider.execution_id, provider._cache)
 
     return lazy_extended_fn
 
@@ -152,30 +129,22 @@ class MetricRegistry:
         return self._metrics
 
     def _next_symbol(self, prefix: str = "x") -> sp.Symbol:
-        """Generate the next symbolic metric.
-
-        Args:
-            prefix (str, optional): symbol prefix. Defaults to "x".
-
-        Returns:
-            str:
-        """
+        """Generate next unique symbol name (x_1, x_2, etc.)."""
         with self._mutex:
             self._curr_index += 1
             return sp.Symbol(f"{prefix}_{self._curr_index}")
 
     def get(self, symbol: sp.Symbol | str) -> SymbolicMetric:
-        """Find the first symbol data that matches the given symbol.
+        """Retrieve symbolic metric by symbol.
 
         Args:
-            symbol: Either a sympy Symbol object or a string representation
-                   of the symbol (e.g., "x_1")
+            symbol: Symbol object or string (e.g., "x_1")
 
         Returns:
             SymbolicMetric containing the symbol's metadata
 
         Raises:
-            DQXError: If the symbol is not found
+            DQXError: If symbol not found
         """
         # Convert string to Symbol if needed
         if isinstance(symbol, str):
@@ -187,11 +156,7 @@ class MetricRegistry:
         return self.index[symbol]
 
     def remove(self, symbol: sp.Symbol) -> None:
-        """Remove a symbolic metric from the registry.
-
-        Args:
-            symbol: The symbolic metric to remove
-        """
+        """Remove symbolic metric from registry."""
         # Remove from metrics list
         self._metrics = [sm for sm in self._metrics if sm.symbol != symbol]
 
@@ -200,6 +165,7 @@ class MetricRegistry:
             del self.index[symbol]
 
     def _exists(self, spec: MetricSpec, lag: int, dataset: str) -> sp.Symbol | None:
+        """Check if metric already exists in registry."""
         for sm in self._metrics:
             if sm.metric_spec == spec and sm.lag == lag and sm.dataset == dataset:
                 return sm.symbol
@@ -214,7 +180,7 @@ class MetricRegistry:
         dataset: str | None = None,
         required_metrics: list[sp.Symbol] | None = None,
     ) -> sp.Symbol:
-        """Register a symbolic metric."""
+        """Register new symbolic metric or return existing one."""
         sym = self._next_symbol()
 
         # Check if symbol already exists, returns the existing one
@@ -239,27 +205,13 @@ class MetricRegistry:
         return sym
 
     def collect_symbols(self, key: ResultKey) -> list[SymbolInfo]:
-        """
-        Collect all symbol values with metadata.
-
-        This method retrieves information about all symbols (metrics) that were
-        registered, evaluates them, and returns their values along with metadata.
-        Symbols are sorted by name for consistent ordering.
+        """Evaluate all symbols and return values with metadata.
 
         Args:
-            key: The ResultKey for evaluation context (date and tags)
+            key: ResultKey for evaluation context (date and tags)
 
         Returns:
-            List of SymbolInfo instances, sorted by symbol name in natural numeric
-            order (x_1, x_2, ..., x_10, x_11, etc. rather than lexicographic).
-            Each contains the symbol name, metric description, dataset,
-            computed value, and context information (date, tags).
-
-        Example:
-            >>> symbols = provider.collect_symbols(key)
-            >>> for s in symbols:
-            ...     if s.value.is_success():
-            ...         print(f"{s.metric}: {s.value.unwrap()}")
+            List of SymbolInfo sorted by natural numeric order (x_1, x_2, ..., x_10).
         """
         symbols = []
 
@@ -294,34 +246,122 @@ class MetricRegistry:
 
         return sorted_symbols
 
+    def topological_sort(self) -> None:
+        """Sort metrics by dependencies for correct evaluation order.
+
+        Raises:
+            DQXError: If circular dependency detected.
+        """
+        from collections import deque
+
+        n = len(self._metrics)
+        if n == 0:
+            return
+
+        # Build dependency graph
+        in_degree: dict[sp.Symbol, int] = {}
+        adjacency: dict[sp.Symbol, list[sp.Symbol]] = {}
+
+        # Initialize structures
+        for sm in self._metrics:
+            # Only count dependencies that are actually in the registry
+            internal_deps = [req for req in sm.required_metrics if req in self._symbol_index]
+            in_degree[sm.symbol] = len(internal_deps)
+            adjacency[sm.symbol] = []
+
+        # Build reverse adjacency (who depends on me?)
+        for sm in self._metrics:
+            for req_symbol in sm.required_metrics:
+                if req_symbol in adjacency:  # Only if required metric is in current list
+                    adjacency[req_symbol].append(sm.symbol)
+
+        # Initialize queue with metrics having no dependencies
+        queue: deque[SymbolicMetric] = deque(sm for sm in self._metrics if in_degree[sm.symbol] == 0)
+        result: list[SymbolicMetric] = []
+
+        # Process metrics in topological order
+        while queue:
+            current = queue.popleft()
+            result.append(current)
+
+            # Reduce in-degree for dependent metrics
+            for dependent_symbol in adjacency[current.symbol]:
+                in_degree[dependent_symbol] -= 1
+                if in_degree[dependent_symbol] == 0:
+                    # Use self.get() to retrieve the metric
+                    queue.append(self.get(dependent_symbol))
+
+        # Check for cycles
+        if len(result) != n:
+            # Find metrics involved in cycle
+            remaining = [sm for sm in self._metrics if sm not in result]
+            cycle_info = self._find_cycle_details(remaining)
+            raise DQXError(f"Circular dependency detected:\n{cycle_info}")
+
+        # Replace _metrics with sorted order
+        self._metrics = result
+
+    def _find_cycle_details(self, remaining_metrics: list[SymbolicMetric]) -> str:
+        """Generate helpful error message about circular dependencies."""
+        cycle_symbols = {sm.symbol for sm in remaining_metrics}
+        details = []
+
+        for sm in remaining_metrics:
+            deps_in_cycle = [str(dep) for dep in sm.required_metrics if dep in cycle_symbols]
+            if deps_in_cycle:
+                details.append(f"  {sm.symbol} ({sm.name}) depends on: {', '.join(deps_in_cycle)}")
+
+        if not details:
+            # No internal cycle dependencies found, might be external
+            details.append("  Metrics depend on symbols not in the registry")
+
+        return "\n".join(details)
+
+    def symbol_lookup_table(self, key: ResultKey) -> dict[MetricKey, sp.Symbol]:
+        """Create mapping from metric keys to symbol names."""
+        symbol_lookup: dict[MetricKey, sp.Symbol] = {}
+        for sym_metric in self.metrics:
+            if sym_metric.dataset is not None:
+                # Calculate effective key based on lag
+                effective_key = key.lag(sym_metric.lag)
+                metric_key = (sym_metric.metric_spec, effective_key, sym_metric.dataset)
+                symbol_lookup[metric_key] = sym_metric.symbol
+        return symbol_lookup
+
 
 class RegistryMixin:
     @property
     def registry(self) -> MetricRegistry:
+        """Access to metric registry."""
         raise NotImplementedError("Subclasses must implement registry property.")
 
     @property
     def metrics(self) -> list[SymbolicMetric]:
+        """Access to all registered metrics."""
         return self.registry._metrics
 
     @property
     def index(self) -> SymbolIndex:
+        """Access to symbol index mapping."""
         return self.registry._symbol_index
 
     def symbols(self) -> Iterable[sp.Symbol]:
+        """Get all registered symbols."""
         return self.registry._symbol_index.keys()
 
     def get_symbol(self, symbol: sp.Symbol | str) -> SymbolicMetric:
+        """Retrieve symbolic metric by symbol."""
         return self.registry.get(symbol)
 
     def remove_symbol(self, symbol: sp.Symbol) -> None:
-        # Remove dependencies recursively
+        """Remove symbol and its dependencies recursively."""
         sm = self.get_symbol(symbol)
         for dep_symbol in sm.required_metrics:
             self.remove_symbol(dep_symbol)  # Recursive removal only
         self.registry.remove(symbol)
 
     def collect_symbols(self, key: ResultKey) -> list[SymbolInfo]:
+        """Evaluate all symbols and return values with metadata."""
         return self.registry.collect_symbols(key)
 
 
@@ -334,38 +374,22 @@ class SymbolicMetricBase(ABC, RegistryMixin):
         return self._registry
 
     def evaluate(self, symbol: sp.Symbol, key: ResultKey) -> Result[float, str]:
-        """
-        Evaluate the given symbolic metric.
-
-        This method takes a symbolic metric and an evaluation context (date and tags)
-        and returns the computed value of the metric along with its metadata.
+        """Evaluate symbol to compute its numeric value.
 
         Args:
             symbol: The symbolic metric to evaluate.
             key: The ResultKey for evaluation context (date and tags).
 
         Returns:
-            Result containing the computed value of the metric along with its metadata.
+            Result containing the computed value or error message.
         """
         return self.index[symbol].fn(key)
 
     def print_symbols(self, key: ResultKey) -> None:
-        """
-        Collect and display all symbol values in a formatted table.
-
-        This is a convenience method that combines collect_symbols() and
-        print_symbols() from the display module into a single call.
+        """Display all symbols in formatted table.
 
         Args:
             key: The ResultKey for evaluation context (date and tags)
-
-        Example:
-            >>> # Instead of:
-            >>> symbols = provider.collect_symbols(key)
-            >>> print_symbols(symbols)
-            >>>
-            >>> # You can now simply do:
-            >>> provider.print_symbols(key)
         """
         from dqx.display import print_symbols
 
@@ -373,33 +397,14 @@ class SymbolicMetricBase(ABC, RegistryMixin):
         print_symbols(symbols)
 
     def build_deduplication_map(self, context_key: ResultKey) -> dict[sp.Symbol, sp.Symbol]:
-        """Build symbol substitution map for deduplication.
-
-        This method identifies duplicate symbols that represent the same metric
-        computed for the same effective date and dataset. It returns a mapping
-        from duplicate symbols to their canonical representatives.
+        """Find duplicate symbols with same metric and effective date.
 
         Args:
-            context_key: The analysis date context. Used to calculate effective
-                        dates for lagged metrics.
+            context_key: Analysis date context for calculating effective dates.
 
         Returns:
-            Dict mapping duplicate symbols to canonical symbols. For example:
-            {
-                sp.Symbol('x_3'): sp.Symbol('x_1'),  # x_3 is duplicate of x_1
-                sp.Symbol('x_5'): sp.Symbol('x_2'),  # x_5 is duplicate of x_2
-            }
-
-            The canonical symbol is always the one with the lowest index number.
-            Empty dict if no duplicates found.
-
-        Example:
-            If we have:
-            - x_1: average(price) for 2024-01-15
-            - x_2: average(price) with lag=1 for 2024-01-16 (effective: 2024-01-15)
-            - x_3: average(price) for 2024-01-15 (duplicate of x_1)
-
-            This returns: {x_3: x_1, x_2: x_1}
+            Dict mapping duplicate symbols to canonical symbols.
+            Canonical symbol is always the one with lowest index number.
         """
         groups: dict[tuple[str, str, str | None], list[sp.Symbol]] = {}
 
@@ -430,7 +435,7 @@ class SymbolicMetricBase(ABC, RegistryMixin):
         return substitutions
 
     def deduplicate_required_metrics(self, substitutions: dict[sp.Symbol, sp.Symbol]) -> None:
-        """Update required_metrics in all symbolic metrics after deduplication.
+        """Update symbol dependencies after deduplication.
 
         Args:
             substitutions: Map of duplicate symbols to canonical symbols
@@ -441,7 +446,7 @@ class SymbolicMetricBase(ABC, RegistryMixin):
                 sym_metric.required_metrics = [substitutions.get(req, req) for req in sym_metric.required_metrics]
 
     def prune_duplicate_symbols(self, substitutions: dict[sp.Symbol, sp.Symbol]) -> None:
-        """Remove duplicate symbols from the provider.
+        """Remove duplicate symbols from registry.
 
         Args:
             substitutions: Map from duplicate symbols to canonical symbols
@@ -467,13 +472,7 @@ class SymbolicMetricBase(ABC, RegistryMixin):
             logger.info("Pruned %d duplicate symbols: %s", len(sorted_symbols), ", ".join(sorted_symbols))
 
     def symbol_deduplication(self, graph: "Graph", context_key: ResultKey) -> None:
-        """Apply symbol deduplication to graph and provider.
-
-        This method:
-        1. Builds a map of duplicate symbols
-        2. Applies deduplication to the graph
-        3. Updates required_metrics references
-        4. Prunes duplicate symbols
+        """Apply full deduplication workflow to graph and registry.
 
         Args:
             graph: The computation graph to apply deduplication to
@@ -506,14 +505,17 @@ class ExtendedMetricProvider(RegistryMixin):
 
     @property
     def provider(self) -> MetricProvider:
+        """Access to parent provider."""
         return self._provider
 
     @property
     def registry(self) -> MetricRegistry:
+        """Access to metric registry from parent provider."""
         return self._provider.registry
 
     @property
     def db(self) -> MetricDB:
+        """Access to metric database from parent provider."""
         return self._provider._db
 
     @property
@@ -522,6 +524,7 @@ class ExtendedMetricProvider(RegistryMixin):
         return self._provider.execution_id
 
     def day_over_day(self, metric: sp.Symbol, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create day-over-day ratio metric (today/yesterday)."""
         # Get the full SymbolicMetric object
         symbolic_metric = self._provider.get_symbol(metric)
         spec = symbolic_metric.metric_spec
@@ -537,12 +540,13 @@ class ExtendedMetricProvider(RegistryMixin):
         fn = _create_lazy_extended_fn(self._provider, compute.day_over_day, spec, sym)
 
         # Register with lazy function
+        cloned_spec = specs.DayOverDay.from_base_spec(spec)
         self.registry._metrics.append(
             sm := SymbolicMetric(
-                name=specs.DayOverDay.from_base_spec(spec).name,
+                name=cloned_spec.name,
                 symbol=sym,
                 fn=fn,
-                metric_spec=specs.DayOverDay.from_base_spec(spec),
+                metric_spec=cloned_spec,
                 lag=lag,  # Use the provided lag instead of 0
                 dataset=dataset,
                 required_metrics=[lag_0, lag_1],
@@ -555,6 +559,7 @@ class ExtendedMetricProvider(RegistryMixin):
         return sym
 
     def week_over_week(self, metric: sp.Symbol, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create week-over-week ratio metric (today/week_ago)."""
         # Get the full SymbolicMetric object
         symbolic_metric = self._provider.get_symbol(metric)
         spec = symbolic_metric.metric_spec
@@ -570,12 +575,13 @@ class ExtendedMetricProvider(RegistryMixin):
         fn = _create_lazy_extended_fn(self._provider, compute.week_over_week, spec, sym)
 
         # Register with lazy function
+        cloned_spec = specs.WeekOverWeek.from_base_spec(spec)
         self.registry._metrics.append(
             sm := SymbolicMetric(
-                name=specs.WeekOverWeek.from_base_spec(spec).name,
+                name=cloned_spec.name,
                 symbol=sym,
                 fn=fn,
-                metric_spec=specs.WeekOverWeek.from_base_spec(spec),
+                metric_spec=cloned_spec,
                 lag=lag,  # Use the provided lag instead of 0
                 dataset=dataset,
                 required_metrics=[lag_0, lag_7],
@@ -588,30 +594,16 @@ class ExtendedMetricProvider(RegistryMixin):
         return sym
 
     def stddev(self, metric: sp.Symbol, offset: int, n: int, dataset: str | None = None) -> sp.Symbol:
-        """Calculate standard deviation of a metric over a window of values.
-
-        This method computes the standard deviation of a metric across n consecutive
-        days, starting from the specified offset.
+        """Create standard deviation metric over time window.
 
         Args:
-            metric: The base metric symbol to calculate standard deviation for.
-            offset: The starting position of the window (0 = today, 1 = yesterday, etc.).
-                    This determines where the window of values begins.
-            n: The number of values to include in the standard deviation calculation.
-               The window will span from offset to offset+n-1.
-            dataset: Optional dataset name. If not provided, uses the dataset from
-                    the base metric.
+            metric: Base metric symbol to calculate standard deviation for.
+            offset: Starting position (0=today, 1=yesterday, etc.).
+            n: Number of values in window (offset to offset+n-1).
+            dataset: Optional dataset name.
 
         Returns:
-            A Symbol representing the standard deviation metric.
-
-        Example:
-            >>> # Calculate stddev of last 7 days starting from today (offset=0)
-            >>> avg_price = provider.average("price")
-            >>> stddev_7d = provider.ext.stddev(avg_price, offset=0, n=7)
-            >>>
-            >>> # Calculate stddev of 5 days starting from 2 days ago (offset=2)
-            >>> stddev_5d = provider.ext.stddev(avg_price, offset=2, n=5)
+            Symbol representing the standard deviation metric.
         """
         # Get the full SymbolicMetric object
         symbolic_metric = self._provider.get_symbol(metric)
@@ -624,21 +616,27 @@ class ExtendedMetricProvider(RegistryMixin):
             required_metric = self.provider.create_metric(spec, lag=i, dataset=symbolic_metric.dataset)
             required.append(required_metric)
 
+        # Generate symbol first
+        sym = self.registry._next_symbol()
+
         # Create lazy function for stddev using lambda to handle the size parameter
         fn = _create_lazy_extended_fn(
             self._provider,
-            lambda db, metric, dataset, key, execution_id: compute.stddev(db, metric, n, dataset, key, execution_id),
+            lambda metric, dataset, key, execution_id, cache: compute.stddev(
+                metric, dataset, key, execution_id, n, cache
+            ),
             spec,
-            sym := self.registry._next_symbol(),
+            sym,
         )
 
         # Register with lazy function
+        cloned_spec = specs.Stddev.from_base_spec(spec, offset, n)
         self.registry._metrics.append(
             sm := SymbolicMetric(
-                name=specs.Stddev.from_base_spec(spec, offset, n).name,
+                name=cloned_spec.name,
                 symbol=sym,
                 fn=fn,
-                metric_spec=specs.Stddev.from_base_spec(spec, offset, n),
+                metric_spec=cloned_spec,
                 lag=offset,  # stddev itself should have lag=offset (not lag=0)
                 dataset=dataset,
                 required_metrics=required,
@@ -657,15 +655,42 @@ class MetricProvider(SymbolicMetricBase):
         self._db = db
         self._execution_id = execution_id
 
+        # Create cache in provider
+        from dqx.cache import MetricCache
+
+        self._cache = MetricCache(db)
+
     @property
     def execution_id(self) -> ExecutionId:
         """The execution ID for this provider instance."""
         return self._execution_id
 
+    @property
+    def cache(self) -> MetricCache:
+        """Access to the metric cache."""
+        return self._cache
+
     @property
     def ext(self) -> ExtendedMetricProvider:
+        """Access to extended metric provider for DoD, WoW, stddev."""
         return ExtendedMetricProvider(self)
 
+    def clear_cache(self) -> None:
+        """Clear the metric cache."""
+        self._cache.clear()
+
+    def flush_cache(self) -> int:
+        """Flush dirty cache entries to DB.
+
+        Returns:
+            Number of metrics flushed
+        """
+        return self._cache.write_back()
+
+    def get_cache_stats(self) -> dict[str, int]:
+        """Get cache statistics."""
+        return {"total_cached": len(self._cache._cache), "dirty_count": self._cache.get_dirty_count()}
+
     def create_metric(
         self,
         metric_spec: MetricSpec,
@@ -689,7 +714,7 @@ class MetricProvider(SymbolicMetricBase):
         Raises:
             ValueError: If the metric type is not supported.
         """
-        if not metric_spec.is_extended:
+        if isinstance(metric_spec, specs.SimpleMetricSpec):
             # Simple metric - use the standard metric method
             return self.metric(metric_spec, lag=lag, dataset=dataset)
 
@@ -716,7 +741,7 @@ class MetricProvider(SymbolicMetricBase):
 
     def metric(
         self,
-        metric: MetricSpec,
+        metric: specs.SimpleMetricSpec,
         lag: int = 0,
         dataset: str | None = None,
     ) -> sp.Symbol:
@@ -738,15 +763,16 @@ class MetricProvider(SymbolicMetricBase):
         sym = self.registry._next_symbol()
 
         # Create lazy retrieval function that will resolve dataset at evaluation time
-        fn = _create_lazy_retrieval_fn(self, metric, sym)
+        cloned_spec = metric.clone()
+        fn = _create_lazy_retrieval_fn(self, cloned_spec, sym)
 
         # Register with the lazy function
         self.registry._metrics.append(
             sm := SymbolicMetric(
-                name=metric.name,
+                name=cloned_spec.name,
                 symbol=sym,
                 fn=fn,
-                metric_spec=metric,
+                metric_spec=cloned_spec,
                 lag=lag,
                 dataset=dataset,
                 required_metrics=[],
@@ -759,30 +785,39 @@ class MetricProvider(SymbolicMetricBase):
         return sym
 
     def num_rows(self, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create metric counting number of rows."""
         return self.metric(specs.NumRows(), lag, dataset)
 
     def first(self, column: str, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create metric returning first value in column."""
         return self.metric(specs.First(column), lag, dataset)
 
     def average(self, column: str, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create metric calculating average of column."""
         return self.metric(specs.Average(column), lag, dataset)
 
     def minimum(self, column: str, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create metric finding minimum value in column."""
         return self.metric(specs.Minimum(column), lag, dataset)
 
     def maximum(self, column: str, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create metric finding maximum value in column."""
         return self.metric(specs.Maximum(column), lag, dataset)
 
     def sum(self, column: str, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create metric summing values in column."""
         return self.metric(specs.Sum(column), lag, dataset)
 
     def null_count(self, column: str, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create metric counting null values in column."""
         return self.metric(specs.NullCount(column), lag, dataset)
 
     def variance(self, column: str, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create metric calculating variance of column."""
         return self.metric(specs.Variance(column), lag, dataset)
 
     def duplicate_count(self, columns: list[str], lag: int = 0, dataset: str | None = None) -> sp.Symbol:
+        """Create metric counting duplicate rows by columns."""
         return self.metric(specs.DuplicateCount(columns), lag, dataset)
 
     @overload
@@ -807,52 +842,23 @@ class MetricProvider(SymbolicMetricBase):
         lag: int = 0,
         dataset: str | None = None,
     ) -> sp.Symbol:
-        """Count occurrences of specific value(s) in a column.
-
-        This operation counts only the specified values, never NULLs.
-        Empty strings are counted as values, not as NULLs.
+        """Count occurrences of specific values in column.
 
         Args:
             column: Column name to count values in
-            values: Value(s) to count - single int/str/bool or list of int/str
+            values: Value(s) to count - single value or list
             lag: Lag offset in days
             dataset: Optional dataset name
 
         Returns:
             Symbol representing the count
-
-        Examples:
-            >>> from dqx import ValidationSuite
-            >>> suite = ValidationSuite("test")
-
-            >>> # Count single value
-            >>> suite.count_values("status", "active")
-
-            >>> # Count multiple values efficiently in one query
-            >>> suite.count_values("status", ["active", "pending"])
-
-            >>> # Count integer values
-            >>> suite.count_values("type_id", [1, 2, 3])
-
-            >>> # Count boolean values
-            >>> suite.count_values("is_active", True)
-            >>> suite.count_values("is_deleted", False)
-
-        Performance Note:
-            Counting multiple values with a list is more efficient than
-            making multiple separate count_values calls.
         """
         return self.metric(specs.CountValues(column, values), lag, dataset)
 
     def unique_count(self, column: str, lag: int = 0, dataset: str | None = None) -> sp.Symbol:
-        """Count unique (distinct) non-null values in a column.
+        """Count distinct non-null values in column.
 
-        This operation counts the number of distinct non-null values in a column.
-        NULL values are not included in the count.
-
-        Note: This metric cannot be merged across partitions. The unique count
-        from multiple partitions cannot be summed to get the total unique count
-        because the same values might appear in multiple partitions.
+        Note: Cannot be merged across partitions.
 
         Args:
             column: Column name to count unique values in
@@ -861,20 +867,65 @@ class MetricProvider(SymbolicMetricBase):
 
         Returns:
             Symbol representing the unique count
-
-        Examples:
-            >>> from dqx import ValidationSuite
-            >>> suite = ValidationSuite("test")
-
-            >>> # Count unique product IDs
-            >>> suite.unique_count("product_id")
-
-            >>> # Count unique users with a 7-day lag
-            >>> suite.unique_count("user_id", lag=7)
-
-        See Also:
-            - duplicate_count: For counting duplicate values
-            - count_values: For counting specific values
-            - null_count: For counting NULL values
         """
         return self.metric(specs.UniqueCount(column), lag, dataset)
+
+    def get_metric(
+        self, metric_spec: MetricSpec, result_key: ResultKey, dataset: str, execution_id: ExecutionId
+    ) -> Result[Metric, str]:
+        """Get metric from cache or database."""
+        # Try cache first
+        cache_result = self._cache.get((metric_spec, result_key, dataset, execution_id))
+        match cache_result:
+            case Some(metric):
+                # Wrap in Result for return type compatibility
+                from returns.result import Success
+
+                return Success(metric)
+            case _:
+                # Cache miss - continue to DB lookup
+                pass
+
+        # Cache miss - get from DB
+        metrics = self._db.get_by_execution_id(execution_id)
+        for metric in metrics:
+            if metric.spec == metric_spec and metric.key == result_key and metric.dataset == dataset:
+                # Populate cache before returning
+                self._cache.put(metric)
+                # Wrap in Result for return type compatibility
+                from returns.result import Success
+
+                return Success(metric)
+
+        # Return failure instead of Nothing
+        return Failure(f"Metric not found: {metric_spec.name} for {dataset}")
+
+    def persist(self, metrics: list[Metric]) -> None:
+        """Save metrics to database and update cache."""
+        # Persist to DB
+        self._db.persist(metrics)
+
+        # Update cache
+        self._cache.put(metrics)
+
+    def get_metrics_by_execution_id(self, execution_id: ExecutionId) -> list[Metric]:
+        """Get all metrics for execution ID using cache."""
+        # First check if we can get all metrics from cache
+        # This is a simple implementation - in production you might want
+        # to track which execution_ids are fully cached
+        metrics_from_db = list(self._db.get_by_execution_id(execution_id))
+
+        # Try to get each metric from cache first
+        result_metrics = []
+        for metric in metrics_from_db:
+            cache_key = (metric.spec, metric.key, metric.dataset, execution_id)
+            cache_result = self._cache.get(cache_key)
+            match cache_result:
+                case Some(cached_metric):
+                    result_metrics.append(cached_metric)
+                case _:
+                    # Not in cache, add it
+                    self._cache.put(metric)
+                    result_metrics.append(metric)
+
+        return result_metrics
