@@ -27,8 +27,10 @@ MetricType = Literal[
 
 @runtime_checkable
 class MetricSpec(Protocol):
+    """Base protocol for all metrics."""
+
     metric_type: MetricType
-    is_extended: bool
+    is_extended: Literal[False] | Literal[True]
 
     @property
     def name(self) -> str: ...
@@ -49,9 +51,25 @@ class MetricSpec(Protocol):
     def __eq__(self, other: Any) -> bool: ...
 
 
-class NumRows:
+@runtime_checkable
+class SimpleMetricSpec(MetricSpec, Protocol):
+    """Protocol for simple metrics that support cloning."""
+
+    is_extended: Literal[False]
+
+    def clone(self) -> Self: ...
+
+
+@runtime_checkable
+class ExtendedMetricSpec(MetricSpec, Protocol):
+    """Protocol for extended metrics that don't support cloning."""
+
+    is_extended: Literal[True]
+
+
+class NumRows(SimpleMetricSpec):
     metric_type: MetricType = "NumRows"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self) -> None:
         self._analyzers = (ops.NumRows(),)
@@ -76,6 +94,10 @@ class NumRows:
     def deserialize(cls, state: bytes) -> states.State:
         return states.SimpleAdditiveState.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__()
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -88,9 +110,9 @@ class NumRows:
         return self.name
 
 
-class First:
+class First(SimpleMetricSpec):
     metric_type: MetricType = "First"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str) -> None:
         self._column = column
@@ -115,6 +137,10 @@ class First:
     def deserialize(cls, state: bytes) -> states.State:
         return states.First.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__(self._column)
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -127,9 +153,9 @@ class First:
         return self.name
 
 
-class Average:
+class Average(SimpleMetricSpec):
     metric_type: MetricType = "Average"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str) -> None:
         self._column = column
@@ -155,6 +181,10 @@ class Average:
     def deserialize(cls, state: bytes) -> states.State:
         return states.Average.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__(self._column)
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -167,9 +197,9 @@ class Average:
         return self.name
 
 
-class Variance:
+class Variance(SimpleMetricSpec):
     metric_type: MetricType = "Variance"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str) -> None:
         self._column = column
@@ -195,6 +225,10 @@ class Variance:
     def deserialize(cls, state: bytes) -> states.State:
         return states.Variance.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__(self._column)
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -207,9 +241,9 @@ class Variance:
         return self.name
 
 
-class Minimum:
+class Minimum(SimpleMetricSpec):
     metric_type: MetricType = "Minimum"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str) -> None:
         self._column = column
@@ -234,6 +268,10 @@ class Minimum:
     def deserialize(cls, state: bytes) -> states.State:
         return states.Minimum.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__(self._column)
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -246,9 +284,9 @@ class Minimum:
         return self.name
 
 
-class Maximum:
+class Maximum(SimpleMetricSpec):
     metric_type: MetricType = "Maximum"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str) -> None:
         self._column = column
@@ -273,6 +311,10 @@ class Maximum:
     def deserialize(cls, state: bytes) -> states.State:
         return states.Maximum.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__(self._column)
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -285,9 +327,9 @@ class Maximum:
         return self.name
 
 
-class Sum:
+class Sum(SimpleMetricSpec):
     metric_type: MetricType = "Sum"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str) -> None:
         self._column = column
@@ -312,6 +354,10 @@ class Sum:
     def deserialize(cls, state: bytes) -> states.State:
         return states.SimpleAdditiveState.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__(self._column)
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -324,9 +370,9 @@ class Sum:
         return self.name
 
 
-class NullCount:
+class NullCount(SimpleMetricSpec):
     metric_type: MetricType = "NullCount"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str) -> None:
         self._column = column
@@ -351,6 +397,10 @@ class NullCount:
     def deserialize(cls, state: bytes) -> states.State:
         return states.SimpleAdditiveState.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__(self._column)
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -363,9 +413,9 @@ class NullCount:
         return self.name
 
 
-class NegativeCount:
+class NegativeCount(SimpleMetricSpec):
     metric_type: MetricType = "NegativeCount"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str) -> None:
         self._column = column
@@ -390,6 +440,10 @@ class NegativeCount:
     def deserialize(cls, state: bytes) -> states.State:
         return states.SimpleAdditiveState.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__(self._column)
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -402,9 +456,9 @@ class NegativeCount:
         return self.name
 
 
-class DuplicateCount:
+class DuplicateCount(SimpleMetricSpec):
     metric_type: MetricType = "DuplicateCount"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, columns: list[str]) -> None:
         if not columns:
@@ -433,6 +487,11 @@ class DuplicateCount:
     def deserialize(cls, state: bytes) -> states.State:
         return states.NonMergeable.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        # Pass a copy of the columns list to avoid shared references
+        return self.__class__(self._columns.copy())
+
     def __hash__(self) -> int:
         # Convert the columns list to a tuple for hashing
         return hash((self.name, tuple(self._columns)))
@@ -446,9 +505,9 @@ class DuplicateCount:
         return self.name
 
 
-class CountValues:
+class CountValues(SimpleMetricSpec):
     metric_type: MetricType = "CountValues"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str, values: int | str | bool | list[int] | list[str]) -> None:
         self._column = column
@@ -476,6 +535,14 @@ class CountValues:
     def deserialize(cls, state: bytes) -> states.State:
         return states.SimpleAdditiveState.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        # If values is a list, create a copy to avoid shared references
+        if isinstance(self._values, list):
+            return self.__class__(self._column, self._values.copy())
+        else:
+            return self.__class__(self._column, self._values)
+
     def __hash__(self) -> int:
         # Convert lists to tuples for hashing
         hashable_values = self._values if not isinstance(self._values, list) else tuple(self._values)
@@ -490,9 +557,9 @@ class CountValues:
         return self.name
 
 
-class UniqueCount:
+class UniqueCount(SimpleMetricSpec):
     metric_type: MetricType = "UniqueCount"
-    is_extended: bool = False
+    is_extended: Literal[False] = False
 
     def __init__(self, column: str) -> None:
         self._column = column
@@ -517,6 +584,10 @@ class UniqueCount:
     def deserialize(cls, state: bytes) -> states.State:
         return states.NonMergeable.deserialize(state)
 
+    def clone(self) -> Self:
+        """Create a new instance with the same parameters but new analyzer prefixes."""
+        return self.__class__(self._column)
+
     def __hash__(self) -> int:
         return hash((self.name, tuple(self.parameters.items())))
 
@@ -529,9 +600,9 @@ class UniqueCount:
         return self.name
 
 
-class DayOverDay:
+class DayOverDay(ExtendedMetricSpec):
     metric_type: MetricType = "DayOverDay"
-    is_extended: bool = True
+    is_extended: Literal[True] = True
 
     def __init__(self, base_metric_type: str, base_parameters: dict[str, Any]) -> None:
         self._base_metric_type = base_metric_type
@@ -566,12 +637,12 @@ class DayOverDay:
     def analyzers(self) -> Sequence[ops.Op]:
         return ()
 
-    def state(self) -> states.SimpleAdditiveState:
-        return states.SimpleAdditiveState(value=0.0)
+    def state(self) -> states.NonMergeable:
+        return states.NonMergeable(value=0.0, metric_type="DayOverDay")
 
     @classmethod
     def deserialize(cls, state: bytes) -> states.State:
-        return states.SimpleAdditiveState.deserialize(state)
+        return states.NonMergeable.deserialize(state)
 
     def __hash__(self) -> int:
         # Use base_spec which handles nested hashing properly
@@ -587,9 +658,9 @@ class DayOverDay:
         return self.name
 
 
-class WeekOverWeek:
+class WeekOverWeek(ExtendedMetricSpec):
     metric_type: MetricType = "WeekOverWeek"
-    is_extended: bool = True
+    is_extended: Literal[True] = True
 
     def __init__(self, base_metric_type: str, base_parameters: dict[str, Any]) -> None:
         self._base_metric_type = base_metric_type
@@ -624,12 +695,12 @@ class WeekOverWeek:
     def analyzers(self) -> Sequence[ops.Op]:
         return ()
 
-    def state(self) -> states.SimpleAdditiveState:
-        return states.SimpleAdditiveState(value=0.0)
+    def state(self) -> states.NonMergeable:
+        return states.NonMergeable(value=0.0, metric_type="WeekOverWeek")
 
     @classmethod
     def deserialize(cls, state: bytes) -> states.State:
-        return states.SimpleAdditiveState.deserialize(state)
+        return states.NonMergeable.deserialize(state)
 
     def __hash__(self) -> int:
         # Use base_spec which handles nested hashing properly
@@ -645,9 +716,9 @@ class WeekOverWeek:
         return self.name
 
 
-class Stddev:
+class Stddev(ExtendedMetricSpec):
     metric_type: MetricType = "Stddev"
-    is_extended: bool = True
+    is_extended: Literal[True] = True
 
     def __init__(self, base_metric_type: str, base_parameters: dict[str, Any], offset: int, n: int) -> None:
         self._base_metric_type = base_metric_type
@@ -691,12 +762,12 @@ class Stddev:
     def analyzers(self) -> Sequence[ops.Op]:
         return ()
 
-    def state(self) -> states.SimpleAdditiveState:
-        return states.SimpleAdditiveState(value=0.0)
+    def state(self) -> states.NonMergeable:
+        return states.NonMergeable(value=0.0, metric_type="Stddev")
 
     @classmethod
     def deserialize(cls, state: bytes) -> states.State:
-        return states.SimpleAdditiveState.deserialize(state)
+        return states.NonMergeable.deserialize(state)
 
     def __hash__(self) -> int:
         # Use base_spec which handles nested hashing properly
